<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Service\CRUD;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Service\Sm00013000GeracaoPropostaService;

class Sm00013000GeracaoPropostaController extends SMCrud {
    
    protected $dataSourceName = "fct_geracaoproposta_modelo";
    protected $geracaoPropostaService;
    
    public function __construct(CRUD $crudService, Environment $environment, Sm00013000GeracaoPropostaService $geracaoPropostaService) {
        parent::__construct($crudService, $environment);
        $this->geracaoPropostaService = $geracaoPropostaService;
    }
    
    public function find(Request\Filter $request, Response $response) {
        try {
            parent::find($request, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
            
            $rows = $response->getDataSets()[0]->getRows();
            foreach ($rows as &$row) {
                $row["NRTIPOORCAMENTO"] = Utilities::getArrayDeString($row["NRTIPOORCAMENTO"]);
                //$row["NMTIPOORCAMENTO"] = Utilities::getArrayDeString($row["NMTIPOORCAMENTO"], false, ";");
            }
            unset($row);
            
            $response->setDataSets(array(new DataSet($this->dataSourceName, $rows)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function save(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            parent::save($request, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
            
            if (isset($row["MUDOU_ESCOPOS"]) && $row["MUDOU_ESCOPOS"] == true) {
                $cdoperador = $this->environment->getCdOperador();
                $nrorg = $this->environment->getNrOrg();
                
                if ($row["__is_new"]) {
                    $columnsToReturn = $this->crudService->getColumnsToReturn();
                    $nrgeracaoproposta = $columnsToReturn[0]["NRGERACAOPROPOSTA"];
                }
                else {
                    $nrgeracaoproposta = $row["NRGERACAOPROPOSTA"];
                }
                
                $escopos = $row["NRTIPOORCAMENTO"];
                $this->geracaoPropostaService->salvarEscoposGeracaoProposta($nrgeracaoproposta, $escopos, $cdoperador, $nrorg);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function inserirComposicoesElementosMedidas(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrgeracaoproposta = $row["NRGERACAOPROPOSTA"];
            $elementos         = $row["ELEMENTOS"];
            $composicoes       = $row["COMPOSICOES"];
            $nrEscopos         = $row["ESCOPOS"];
            
            $cdoperador = $this->environment->getCdOperador();
            $nrorg = $this->environment->getNrOrg();
            
            $this->geracaoPropostaService->inserirComposicoesElementosMedidas($nrgeracaoproposta, $elementos, $composicoes, $nrEscopos, $cdoperador, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarJsonIntegracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrgeracaoproposta = $row["NRGERACAOPROPOSTA"];
            $nrorg = $this->environment->getNrOrg();
            
            $jsonIntegracao = $this->geracaoPropostaService->gerarJsonIntegracao($nrgeracaoproposta, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("json" => $jsonIntegracao)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}