<?php
namespace Teknisa\Model\Geral\Repositories;
use Teknisa\Model\Traits\TQuery;

class Operador extends \Doctrine\ORM\EntityRepository {

    use TQuery;

    public function getOperatorByNmAccess($nmAccess){
        $query = "SELECT CDOPERADOR, NMOPERADOR
                    FROM OPERADOR
                   WHERE UPPER(NMACESSO) = UPPER(:NMACESSO)";

        $params = array( 'NMACESSO' => $nmAccess );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    public static function verificaGrupoAlteraPonto($cdoperador){
        $sql = "SELECT COUNT(*) AS GRUPODPAP
                   FROM OPERADOR OP, OPERGRUPOP OG, AUTGRUPOP AG
                  WHERE OP.CDOPERADOR   = :P_CDOPERADOR
                    AND OP.CDOPERADOR   = OG.CDOPERADOR
                    AND OG.CDGRUPOPER   = AG.CDGRUPOPER
                    AND AG.IDAUTGRUPOP  = 'DPAP'";

        $param = array(
            'P_CDOPERADOR' => $cdoperador
        );

        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $param);
        return count($result)>0 ? $result : array();
    }

    private function generateQueryPaginated($query, $startPos, $finalPos)
    {
        if ($startPos && $finalPos){
            $query = <<<SQL
          SELECT *
            FROM (
                SELECT Q.*, rowNum rn
                 FROM ($query) Q
                WHERE rowNum <= $finalPos)
           WHERE rn >= $startPos
SQL;
        }
        return $query;
    }

    public function findAllOperador($cdCliente, $filter, $startPos=0, $finalPos=0)
    {
        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, DSEMAILOPER, NMRAZSOCCLIE, CDCLIENTE, DECODE(CDCLIENTE, :P_CDCLIENTE, 'Y', 'N') AS IDEDITABLE
          FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                  FROM OPERADOR OP
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 UNION
                SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                  FROM OPERCLIE OC
                  JOIN OPERADOR OP ON (OP.CDOPERADOR = OC.CDOPERADOR)
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                        )
                 WHERE 1 = 1
                 $filter
                 ORDER BY NMOPERADOR
SQL;

        $param = array(
            'P_CDCLIENTE' => $cdCliente
        );

        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findAllOperadorResponsible($filter, $filterTeknisa, $finalPos = 0, $startPos = 0){
        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, NMRAZSOCCLIE, CDCLIENTE
        FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                      FROM OPERADOR OP
                      JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                      AND OP.IDOPERATIVO = 'A'
                      AND CL.IDATIVO = 'S'
        WHERE 1=1
        $filterTeknisa)
        WHERE 1=1
        $filter
        ORDER BY NMOPERADOR

SQL;
        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function findAllOperadorRelationshipCliente($cdCliente, $userId, $filter, $startPos=0, $finalPos=0)
    {

        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, DSEMAILOPER, NMRAZSOCCLIE, CDCLIENTE, DECODE(CDCLIENTE, :P_CDCLIENTE, 'Y', 'N') AS IDEDITABLE,NMACESSO
          FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE,OP.NMACESSO
                  FROM OPERADOR OP
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 WHERE OP.CDCLIENTE = :P_CDCLIENTE
                 UNION
                SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE,OP.NMACESSO
                  FROM OPERCLIE OC
                  JOIN OPERADOR OP ON (OP.CDOPERADOR = OC.CDOPERADOR)
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 WHERE OC.CDCLIENTE = :P_CDCLIENTE
                   AND EXISTS (SELECT O.CDOPERADOR
                                 FROM OPERADOR O
                                WHERE O.CDOPERADOR = :P_CDOPERADOR
                                  AND O.CDCLIENTE <> :P_CDCLIENTE))
                 WHERE 1 = 1
                 $filter
                 ORDER BY NMOPERADOR
SQL;

        $param = array(
            'P_CDCLIENTE' => $cdCliente,
            'P_CDOPERADOR' => $userId,
        );

        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

     public function findAvailableCliente($userId, $cdCliente, $filter = ''){
          $sql = <<<SQL
          SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OCL.CDCLIENTE <> :CD_CLIENTE
                   UNION
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERADOR OPE
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OPE.CDOPERADOR = :CD_OPERADOR AND OPE.CDCLIENTE <> :CD_CLIENTE)
           WHERE 1 = 1
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findAvailableClienteForOperatorMaster($userId, $cdCliente, $filter = '', $limit, $offset){
        $sql = <<<SQL
          SELECT NMFANTCLIE, NMRAZSOCCLIE, CDCLIENTE, NRINSJURCLIE, CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM CLIENTE CLI
                   WHERE CLI.IDCLIEATIV = 'S')
           WHERE 1 = 1
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        $query  = self::getPaginatedQuery($sql, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findClientForSupportOperator($userId, $cdCliente, $filter = ''){
        $sql = <<<SQL
          SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S')
                    WHERE OCL.CDCLIENTE <> :CD_CLIENTE
                   )
           WHERE 1 = 1
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findClientsForOperatorMasterAndSupportOperator($userId, $cdCliente, $filter = '', $limit, $offset){
        $sql = <<<SQL
          SELECT NMFANTCLIE, NMRAZSOCCLIE, CDCLIENTE, NRINSJURCLIE, CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM CLIENTE CLI
                   WHERE CLI.IDCLIEATIV = 'S'
                   AND CLI.CDCLIENTE <> :CD_CLIENTE)
           WHERE 1 = 1
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        $query  = self::getPaginatedQuery($sql, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findIfCurrenctClienteStillAvaiable($userId, $cdCliente){
          $sql = <<<SQL
          SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OCL.CDCLIENTE = :CD_CLIENTE
                   UNION
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERADOR OPE
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OPE.CDOPERADOR = :CD_OPERADOR AND OPE.CDCLIENTE = :CD_CLIENTE)
           WHERE 1 = 1
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function getPerfAcessParcByNrorgAndCdoperador($nrOrg, $cdoperador){
        $query = "SELECT PAC.NRPERFILACESSO AS ID, PAC.IDPERFILACESSOADM
                    FROM CP_PERFILACESSO PAC
                    JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                   WHERE PAC.NRORG = :NRORG
                     AND PER.CDOPERADOR = :CDOPERADOR
                     AND PAC.IDATIVO = 'S'";

        $params = array( 'NRORG' => $nrOrg, 'CDOPERADOR' => $cdoperador );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getPerfAcessParcByNrorgAndCdoperadorFull($nrOrg, $user){
        $query = "SELECT PAC.NRPERFILACESSO AS ID, PAC.IDPERFILACESSOADM
                    FROM CP_PERFILACESSO PAC
                    JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                   WHERE PAC.NRORG = :NRORG
                     AND PER.CDOPERADOR = :CDOPERADOR
                     AND PAC.IDATIVO = 'S'
               UNION ALL
                  SELECT PP.NRPERFILACESSO AS ID, PA.IDPERFILACESSOADM
                    FROM PERFACESSPARC PP
                    JOIN PERFILACESSO PA ON PA.NRPERFILACESSO = PP.NRPERFILACESSO
                    JOIN OPERADOR OPE ON (OPE.NRPARCNEGOCIO = PP.NRPARCNEGOCIO)
                   WHERE PP.NRORG = :NRORG
                     AND OPE.CDOPERADOR = :CDOPERADOR
                     AND PP.IDATIVO = 'S'";

        $params = array( 'NRORG' => $nrOrg, 'CDOPERADOR' => $user );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $operador
     * @return array
     */
    public function getOperatorByNmAccessOrEmail($operador) {
        $query = "SELECT CDOPERADOR AS ID, NMOPERADOR AS NAME, NRORG AS ORGANIZATION_ID, DSEMAILOPER AS EMAIL,
                         CDSENHAOPERWEB AS PASSWORD_MD5, CDSENHOPER AS PASSWORD_CRYPTO_PLUS,CDCLIENTE,
                         CASE WHEN IDOPERATIVO = 'A'
                              THEN 'Y'
                              ELSE 'N'
                          END AS IS_ACTIVE,
                         IDIDIOMA, NMACESSO, NMCOMPLOPER, CDEMPRESA, NRPARCNEGOCIO,
                         NRORG, NRORGTRAB, NRESTRUTTRAB, CDFILIALSELECIONADA,
                         CDSENHAOPERSUP, NRORGSUP, TO_CHAR(DTLIMACESSOSUP, 'DD/MM/YYYY HH24:MI:SS') AS DTLIMACESSOSUP
                    FROM OPERADOR
                   WHERE UPPER(NMACESSO) = UPPER(:OPERADOR)
                      OR DSEMAILOPER = :OPERADOR";

        $params = array( 'OPERADOR' => $operador );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    /**
     * @param $operador
     * @return array
     */
    public function getOperatorByNmAccessOrEmailSupportOperator($operador) {
        $query = "SELECT '#SUP_TEKNISA' AS ID, 'Operador de Suporte - Teknisa' AS NAME, NMOPERADOR AS SUP_NAME,
                         NRORGSUP AS ORGANIZATION_ID, 'operador_sup_teknisa9999@teknisa.com' AS EMAIL,
                         CDSENHAOPERSUP AS PASSWORD_MD5, CDSENHOPER AS PASSWORD_CRYPTO_PLUS, CDCLIENTE,
                         DECODE(IDOPERATIVO, 'A', 'Y', 'N') AS IS_ACTIVE, CDOPERADOR AS OPER_SUP_ID,
                         IDIDIOMA, NMACESSO, NMCOMPLOPER, CDEMPRESA, NRPARCNEGOCIO,
                         NRORGSUP AS NRORG, NRORGSUP AS NRORGTRAB, NRESTRUTTRAB, CDFILIALSELECIONADA,
                         CDSENHAOPERSUP, NRORGSUP, TO_CHAR(DTLIMACESSOSUP, 'DD/MM/YYYY HH24:MI:SS') AS DTLIMACESSOSUP
                    FROM OPERADOR
                   WHERE (UPPER(NMACESSO) = UPPER(:OPERADOR)
                          OR DSEMAILOPER = :OPERADOR)
                     AND DTLIMACESSOSUP > SYSDATE";

        $params = array( 'OPERADOR' => $operador );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    public function currentCliente($cdCliente){
        $sql = <<<SQL
               SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,OMG.IDALLOWPRODUCTCALL
                FROM CLIENTE CLI
                JOIN ORGANIZACAO ORG ON(ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO)
           LEFT JOIN OM_PARAM_ORG OMG ON (OMG.NRORG = ORG.NRORG)
               WHERE CLI.CDCLIENTE = :CD_CLIENTE
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findProfileForAddProfile($nrparcnegocio){
          $sql = <<<SQL
                 SELECT PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
              FROM PERFILACESSO PAC
         LEFT JOIN PERFACESSPARC PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
             WHERE PAC.IDATIVO = 'S' AND
             PAC.NRPERFILACESSO NOT IN(
                       SELECT PAC.NRPERFILACESSO
                        FROM PERFILACESSO PAC
                   LEFT JOIN PERFACESSPARC PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                       WHERE (PER.NRPARCNEGOCIO = :NRPARCNEGOCIO)
                    GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
                   )
          GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
          ORDER BY PAC.NMPERFILACESSO
SQL;

        $param = array(
            'NRPARCNEGOCIO' => $nrparcnegocio
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function setEntityManager($em) {
        $this->_em = $em;
    }

    public function updateOrgOperator($dbUser, $email, $nmAccess, $name, $orgId, $code, $entityManager = null) {
        $query = "
        UPDATE OPERADOR
           SET DSEMAILOPER = :DSEMAILOPER,
               NMACESSO    = :NMACESSO,
               NMOPERADOR  = :NMOPERADOR
         WHERE NRORG      = :NRORG
           AND CDOPERADOR = :CDOPERADOR
        ";
        $param = array(
            "DSEMAILOPER" => $email,
            "NMACESSO" => $nmAccess,
            "NMOPERADOR" => $name,
            "NRORG" => $orgId,
            "CDOPERADOR" => $code
        );
        $entityManager = $entityManager ? $entityManager : $this->getEntityManager();
        $entityManager->getConnection()->executeQuery($query, $param);
    }

    public function updateSupportOperator($cdoperador, $nrorg, $cdsenhaopersup, $dtlimacessosup, $nrorgsup) {
        $date = $dtlimacessosup ? $dtlimacessosup->format('d/m/Y H:i:s') : null;
        $query = "
        UPDATE OPERADOR
           SET NRORGSUP        = :NRORGSUP,
               CDSENHAOPERSUP  = :CDSENHAOPERSUP,
               DTLIMACESSOSUP  = TO_DATE('$date','DD/MM/YYYY HH24:MI:SS')
         WHERE NRORG           = :NRORG
           AND CDOPERADOR      = :CDOPERADOR
        ";
        $params = array(
            "NRORGSUP" => $nrorgsup,
            "CDSENHAOPERSUP" => $cdsenhaopersup,
            "NRORG" => $nrorg,
            "CDOPERADOR" => $cdoperador
        );
        try {
            $std = $this->getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }
        catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function getDataSupportOperator($cdoperador, $nrorg)
    {
        $sql = <<<SQL
                 SELECT CDOPERADOR, NMOPERADOR, NMACESSO, DSEMAILOPER, NRORG,
                        CDSENHAOPERSUP, NRORGSUP, DTLIMACESSOSUP
                   FROM OPERADOR
                  WHERE NRORG = :NRORG
                    AND CDOPERADOR = :CDOPERADOR
SQL;

        $param = array( "NRORG" => $nrorg, "CDOPERADOR" => $cdoperador );
        $data = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return empty($data) ? null : $data[0];
    }

    public function getCurrentClientSupportOperator($nrorg, $cdoperador)
    {
        $sql = <<<SQL
                 SELECT C.CDCLIENTE, C.NMRAZSOCCLIE, C.NMFANTCLIE, OG.NMORGANIZACAO,
                        TO_CHAR(OP.DTLIMACESSOSUP, 'DD/MM/YYYY HH24:MI') AS EXPIRATION_DATE
                   FROM OPERADOR OP
                   JOIN ORGANIZACAO OG ON OG.NRORG = :NRORG
                   JOIN CLIENTE C ON C.NRPARCNEGOCIO = OG.NRPARCNEGOCIO
                  WHERE OP.CDOPERADOR = :CDOPERADOR
SQL;

        $param = array(
            "NRORG"      => $nrorg,
            "CDOPERADOR" => $cdoperador
        );
        $data = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return empty($data) ? null : $data[0];
    }

    public function saveLogopera($tableName, $currentDate, $nmOperador, $idtipooper, $dsoperacao,
                                 $dsrdmusuoper, $dsendusuoper, $dsnmusuoper, $dsmetusuoper, $cdoperador, $cdfillogoper,
                                 $nrorg, $idativo)
    {
        $query = "
        INSERT INTO LOGOPERA
                    (NMTABELA, DTOPERACAO, NMRESPOPER, IDTIPOOPER, DSOPERACAO, CDPESQITEM, DSRDMUSUOPER, DSENDUSUOPER,
                     DSNMUSUOPER, DSMETUSUOPER, CDORESPOPER, CDFILLOGOPER, NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
                     CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
             VALUES (:NMTABELA, TO_DATE('$currentDate','DD/MM/YYYY HH24:MI:SS'), :NMRESPOPER, :IDTIPOOPER, :DSOPERACAO, :CDPESQITEM, :DSRDMUSUOPER,
                     :DSENDUSUOPER, :DSNMUSUOPER, :DSMETUSUOPER, :CDORESPOPER, :CDFILLOGOPER, :NRORG, TO_DATE('$currentDate','DD/MM/YYYY HH24:MI:SS'),
                     :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU, :IDATIVO)
        ";
        $params = array(
            "NMTABELA" => $tableName,
            "NMRESPOPER" => $nmOperador,
            "IDTIPOOPER" => $idtipooper,
            "DSOPERACAO" => $dsoperacao,
            "CDPESQITEM" => $cdoperador,
            "DSRDMUSUOPER" => $dsrdmusuoper,
            "DSENDUSUOPER" => $dsendusuoper,
            "DSNMUSUOPER" => $dsnmusuoper,
            "DSMETUSUOPER" => $dsmetusuoper,
            "CDORESPOPER" => $cdoperador,
            "CDFILLOGOPER" => $cdfillogoper,
            "NRORG" => $nrorg,
            "DTULTATU" => null,
            "CDOPERINCLUSAO" => $cdoperador,
            "NRORGINCLUSAO" => $nrorg,
            "NRORGULTATU" => $nrorg,
            "CDOPERULTATU" => null,
            "IDATIVO" => $idativo,
        );
        try {
            $std = $this->getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }
        catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findOrgOperator($orgOperatorEmail, $orgId, $entityManager = null){
        $sql = <<<SQL
                 SELECT CDOPERADOR, NMOPERADOR, NMACESSO, DSEMAILOPER
                   FROM OPERADOR
                  WHERE NRORG = :NRORG
                    AND DSEMAILOPER = :EMAIL
SQL;

        $param = array( "NRORG" => $orgId, "EMAIL" => $orgOperatorEmail );
        $entityManager = $entityManager ? $entityManager : $this->getEntityManager();
        return $entityManager->getConnection()->fetchAll($sql, $param);
    }

    public function findProfileForAddProfileCP($cdCliente,$cdoperador){
        $sql = <<<SQL
               SELECT PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
              FROM CP_PERFILACESSO PAC
         LEFT JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
             WHERE PAC.IDATIVO = 'S' AND
             PAC.NRPERFILACESSO NOT IN(
                       SELECT PAC.NRPERFILACESSO
                        FROM CP_PERFILACESSO PAC
                   LEFT JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                       WHERE (PER.CDOPERADOR = :CDOPERADOR AND PAC.CDCLIENTE = :CDCLIENTE)
                    GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
                   )
          GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
          ORDER BY PAC.NMPERFILACESSO
SQL;

        $param = array(
            'CDOPERADOR' => $cdoperador,
            'CDCLIENTE' => $cdCliente,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findAllOperadores(){
        $sql = <<<SQL
               SELECT DISTINCT OP.CDOPERADOR, NMOPERADOR,
                      DECODE(IDOPERMASTERCP, 'N', 'Não', 'S', 'Sim') AS IDOPERMASTERCP,
                      DECODE(IDOPERMASTERZMART, 'N', 'Não', 'S', 'Sim') AS IDOPERMASTERZMART
                 FROM OPERADOR OP
                WHERE OP.IDOPERATIVO <> 'I'
             ORDER BY OP.NMOPERADOR
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql);
    }

    public function findOperadoresPorCliente($cdCliente){
        $sql = <<<SQL
               SELECT OC.CDOPERADOR, OP.NMOPERADOR, OC.NRPRIOPERCLI,
                      OC.CDCLIENTE, OC.IDEMAILENCSO,
                      DECODE(OC.IDTPOPERCLIE, 'R', 'Responsável', 'C', 'Cliente') AS IDTPOPERCLIE
                 FROM OPERADOR OP, OPERCLIE OC
                WHERE OP.CDOPERADOR = OC.CDOPERADOR
                  AND OC.CDCLIENTE = :P_CDCLIENTE
             ORDER BY OP.NMOPERADOR
SQL;
        $param = array(
            'P_CDCLIENTE' => $cdCliente,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $param);
    }

    public function findRelacionarOperadoresComCliente($cdcliente){
        $sql = <<<SQL
               SELECT O.CDOPERADOR, O.NMOPERADOR
                 FROM OPERADOR O
                WHERE O.IDOPERATIVO = 'A'
                  AND O.CDOPERADOR NOT IN (SELECT OC.CDOPERADOR
                                             FROM OPERADOR OP, OPERCLIE OC
                                            WHERE OP.CDOPERADOR = OC.CDOPERADOR
                                              AND OC.CDCLIENTE = :P_CDCLIENTE)
            ORDER BY O.NMOPERADOR
SQL;
        $param = array(
            'P_CDCLIENTE' => $cdcliente,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $param);
    }

    public function findOperadorByCdoperador($cdoperador){
        $sql = <<<SQL
               SELECT CDOPERADOR, NMOPERADOR, IDOPERATIVO, NRORG, CDSENHAOPERWEB, CDSENHOPER, IDIDIOMA,
                         NMACESSO, NMCOMPLOPER, CDEMPRESA, NRPARCNEGOCIO, NRORG, NRORGTRAB,
                         NRESTRUTTRAB, DSEMAILOPER, CDFILIALSELECIONADA, NRORG AS ORGANIZATION_ID, DSEMAILOPER AS EMAIL,
                         CDSENHAOPERWEB AS PASSWORD_MD5, CDSENHOPER AS PASSWORD_CRYPTO_PLUS,
                         CDSENHAOPERSUP, NRORGSUP, TO_CHAR(DTLIMACESSOSUP, 'DD/MM/YYYY HH24:MI:SS') AS DTLIMACESSOSUP
                    FROM OPERADOR
                   WHERE CDOPERADOR = :CDOPERADOR
SQL;
        $param = array(
            'CDOPERADOR' => $cdoperador
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return !empty($result) ? $result[0] : $result;
    }



    public static function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $limit . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $limit;
            }
        }
        return $query;
    }
}
